/*  atoken.cc -- The -*- C++ -*- alphabetic token methods.
    Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
    This file is part of the C++ Alphabetic Data Management Library. */

#include "atoken.h"

// ======
// ATOKEN
// ======

// Determine if the text between tb & te is a token, the limits are
// not verified (they can be valid token symbols).
bool atoken::is(const astring &t, const_iterator tb,
		 const_iterator te) const {

  iterator ti = (iterator) tb;

  // Verify that: [ t.begin() <= tb < te <= t.end() ]
  if ((tb - t.begin() >= 0) && (te - tb > 0) && (t.end() - te >= 0)) {
    
    while (ti != te) {
      if (is_delim(*ti))
	return false;
      ti++;
    }
    
    return true; // we've found a token

  } else {

    return false;

  }

};

// Return the beginning of a token in t starting BEFORE ti
// If ti is not pointing to the text return ti
astring::iterator atoken::begin(const astring &t, const_iterator ti) const {
  
  iterator tj= iterator(ti);
  
  // Verify that: [ t.begin() < ti < t.end() ]
  if ( ( tj - t.begin() > 0 ) && ( tj - t.end() < 0) ) {
    
    while (tj != t.begin()) {
      tj--;
      if (is_predelim(*tj)) {
	return (tj + 1); // the token starts on the next char
      }
    } // end while 
  } // end if
  
  // Special Case, 'tj' == t.begin()
  if (tj == t.begin() ) {
    return iterator(tj);   // ti is t.begin() and the first token symbol
  } else {
    return iterator(ti);     // ERROR, ti < t.begin() || ti >= t.end() 
  }
};

// Return the end of a token in t starting IN ti
// If ti is not pointing to the text return ti
astring::iterator atoken::end(const astring &t, const_iterator ti) const {
  
  // Verify that: [ t.begin() < ti <= t.end() ]
  if (t.begin() - ti < 0 || ti - t.end() <= 0) {
    
    iterator tj = iterator(ti);
    
    while (tj != t.end()) {
      if (is_postdelim(*tj))
	return tj;
      tj++;
    }
    
    return tj;   // tj == t.end()
  }
  
  return iterator(ti);     // ERROR, ti < t.begin() || ti > t.end()
};
