// bmhskm.cc -- The -*- C++ -*- Boyer Moore Horspool skm methods.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#include "bmhskm.h"
#include <iomanip.h> // setw()

/**

  BMH_SKM

**/

// PREPROCESSOR & MATCHER
#define BMHSKM_CC

#define  PREPROCESSOR preprocess        // Preprocess function
#define  MATCHER      match             // Match Function
#include "bmhskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER

#define  BMHSKM_RUN
#define  PREPROCESSOR run_preprocessor  // Run Preprocessor
#define  MATCHER      run_matcher       // Run Matcher
#include "bmhskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER
#undef   BMHSKM_RUN            

#undef BMHSKM_CC            

// print preprocessed data
ostream & bmh_skm::print_pdata (ostream &o) const {

  skm_patt::size_type i;
  skm_patt::iterator  pi;

  // PAT
  o << "pat        : " << *pat << "\n";

  // SKIP
  for (pi=pat_beg; pi != pat_end; pi++)
    o << "skip [\'" << *pi << "\'] : " << skip[aindex(*pi)] << "\n";

  return o;
};

// DISPLAY ANALYSIS
void bmh_skm::algorithm_step(breakpoint WHERE, int count) {

  adev_mpatt mp(pat, pat_last - pat_beg);

  switch (WHERE) {

  case INIT_PREPROCESSOR:
    ad->message("PREPROCESSING ...");
    ad->size_pdata("skip [ * ]", skip[0]);
    break;

  case PREPROCESSOR_STEP:
    if (count == 1) {
      mp.update_pos(pat_last - pat_beg);
      ad->matched_patt(mp);
      string s ("skip [\'"); s += *pat_last; s += "\']";
      ad->size_pdata(s, skip [ aindex(*pat_last) ]);
    }
    break;

  case INIT_MATCHER:
    ad->message("PREPROCESSOR STRUCTURES");
    ad->alpha_size_pdata("skip ", &al, skip, pat_size);
    ad->message("SEARCHING ...");
    ad->matched_text(txt, ti - (pat_size - 1), mp);
    if (*(ti - (pat_size - 1)) != *pat_last ) {
      string s ("skip [\'"); s += *ti; s += "\']";
      ad->size_pdata(s, skip[aindex(*ti)]);
    }
    break;
 
  case MATCHER_STEP:
    if ( txt_end - 1 > ti ) {
      ad->matched_text(txt, ti - (pat_size - 1), mp);
      if (*ti != *pat_last) {
        string s ("skip [\'"); s += *ti; s += "\']";
	ad->size_pdata(s, skip[aindex(*ti)]);
      }
    }
    break;

  case PARTIAL_MATCH:
    ad->message("-- PARTIAL MATCH --");
    break;

  case PARTIAL_MATCH_STEP:
    mp.update_pos(pmpi - pat_beg);
    mp.matched_symbols(pmpi + 1 - pat_beg, pat_size);
    ad->matched_text(txt, ti - (pat_size - 1), mp);
    if (*pmti != *pmpi) {
      string s ("skip [\'"); s += *ti; s += "\']";
      ad->size_pdata(s, skip[aindex(*ti)]);
    }
    break;

  case FULL_MATCH: 
    mp.matched_symbols(0, pat_size);
    ad->matched_text(txt, ti - (pat_size - 1), mp);
    ad->message("-- FULL MATCH --");
    break;

  case MATCH_RECOVERY:
    string s ("skip [\'"); s += *ti; s += "\']";
    ad->size_pdata(s, skip[aindex(*ti)]);
    break;
  };
};

// STATIC OBJECTS
bmh_skm bmhskm;
