// bmskm.cc -- The -*- C++ -*- Boyer Moore skm methods.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#include "bmskm.h"
#include "util.h"    // itos()
#include <iomanip.h> // setw()

/**

  BM_SKM

**/

// PREPROCESSOR & MATCHER
#define BMSKM_CC

#define  PREPROCESSOR preprocess        // Preprocess function
#define  MATCHER      match             // Match Function
#include "bmskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER

#define  BMSKM_RUN
#define  PREPROCESSOR run_preprocessor  // Run Preprocessor
#define  MATCHER      run_matcher       // Run Matcher
#include "bmskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER
#undef   BMSKM_RUN            

#undef BMSKM_CC            

// print preprocessed data
ostream & bm_skm::print_pdata (ostream &o) const {

  skm_patt::size_type i;
  skm_patt::iterator  pi;

  // PAT POSITION
  o << "i          : ";
  for (i=0; i < pat_size ; i++)
    o << setw(3) << i;
  o << "\n";

  // PAT
  pi = pat_beg;
  o << "pat [i]    : ";
  for (i=0; i < pat_size ; i++)
    o << "  " << *pi++;
  o << "\n";

  // SHIFT
  o << "shift [i]  : ";
  for (i=0; i < pat_size; i++)
    o << setw(3) << shift[i];
  o << "\n";

  // SKIP
  for (pi=pat_beg; pi != pat_end; pi++)
    o << "skip [\'" << *pi << "\'] : " << skip[aindex(*pi)] << "\n";
 
  return o;
};


// DISPLAY ANALYSIS
void bm_skm::algorithm_step(breakpoint WHERE, int count) {

  adev_mpatt mp(pat, pat_last - pat_beg);

  switch (WHERE) {

  case INIT_PREPROCESSOR:
    ad->message("PREPROCESSING ...");
    ad->size_pdata("skip [ * ]", skip[0]);
    break;

  case PREPROCESSOR_STEP:
    if (count == 1) {
      if (pat_last != pat_end){
	mp.update_pos(pat_last - pat_beg);
	ad->matched_patt(mp);
	string s("skip  [\'"); s += *pat_last; s += "\']";
	ad->size_pdata(s, skip [ aindex(*pat_last) ]);
        s = "shift ["; s += itos (pat_last - pat_beg); s+= "]";
	ad->size_pdata(s, shift[ pat_last - pat_beg ]);
      } else {
	ad->patt_size_pdata("f", pat, f);
	ad->patt_size_pdata("shift", pat, shift);
      }
    }
    break;

  case INIT_MATCHER:
    ad->message("PREPROCESSOR STRUCTURES");
    ad->patt_size_pdata("shift ", pat, shift);
    ad->alpha_size_pdata("skip ", &al, skip, pat_size);
    ad->message("SEARCHING ...");
    ad->matched_text(txt, ti - (pat_size - 1), mp);
    if (*ti != *pat_last) {
      string s = "max (shift ["; s += itos (pat_size - 1);
      s += "], skip [\'"; s+= *ti; s += "\'])";
      ad->size_pdata(s, max(skip[aindex(*ti)], shift[pat_size - 1]));
    }
    break;
 
  case MATCHER_STEP:
    if ( txt_end - 1 > ti ) {
      ad->matched_text(txt, ti - (pat_size - 1), mp);
      if (*ti != *pat_last) {
        string s = "max (shift ["; s += itos (pat_size - 1);
        s += "], skip [\'"; s+= *ti; s += "\'])";
        ad->size_pdata(s, max(skip[aindex(*ti)], shift[pat_size - 1]));
      }
    }
    break;

  case PARTIAL_MATCH:
    ad->message("-- PARTIAL MATCH --");
    break;

  case PARTIAL_MATCH_STEP:
    mp.update_pos(pmpi - pat_beg);
    mp.matched_symbols(pmpi + 1 - pat_beg, pat_size);
    ad->matched_text(txt, ti - (pmpi - pat_beg), mp);
    if (*ti != *pmpi) {
      string s = "max (shift ["; s += itos (pmpi - pat_beg);
      s += "], skip [\'"; s += *ti; s += "\'])";
      ad->size_pdata(s, max(skip[aindex(*ti)], shift[pmpi - pat_beg]));
    }
    break;

  case FULL_MATCH: 
    mp.matched_symbols(0, pat_size);
    ad->matched_text(txt, ti, mp);
    ad->message("-- FULL MATCH --");
    break;

  case MATCH_RECOVERY:
    string s = "max (shift ["; s += itos (pmpi - pat_beg);
    s += "], skip [\'"; s += *ti; s += "\'])";
    ad->size_pdata(s, max(skip[aindex(*ti)], shift[pmpi - pat_beg]));
    break;
  };
};

// STATIC OBJECTS
bm_skm bmskm;
