// bmskm.h -- The -*- C++ -*- Boyer Moore skm interface.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _BMSKM_H_
#define _BMSKM_H_

#include "skm.h"
#include <vector> // skip table

// =============================================================
// BM SKM (Boyer Moore Single Keyword Matcher) : Class Interface
// =============================================================

class bm_skm : public skm_algo {

protected:

  // internal variables
  vector<astring::size_type> skip;     // Skip Table  (d)
  vector<astring::size_type> shift;    // Shift Table (dd')
  vector<astring::size_type> f;        // Auxiliary Table to build dd'

  skm_patt::iterator         pat_last; // Last pattern symbol iterator
  skm_patt::iterator         pmpi;     // Partial match pattern iterator

public:

  // constructor / destructor
  bm_skm() : skm_algo() {};
  bm_skm(const alpha &a) : skm_algo(a) {};

  ~bm_skm() {};

  // long algorithm name
  virtual 
  const char * name() const { return "Boyer Moore"; };

  // short algorithm name
  virtual 
  const char * id() const { return "bm"; };

  // print preprocesed data 
  virtual 
  ostream & print_pdata(ostream &o) const;

protected:

  virtual 
  void preprocess ();                // preprocess the pattern

  virtual 
  void run_preprocessor();           // run the preprocessor

  virtual 
  astring::size_type match();       // match the pattern

  virtual
  astring::size_type run_matcher(); // run the matcher

  virtual
  void algorithm_step(breakpoint WHERE, int count);
  // algorithm step function
};

// STATIC OBJECTS
extern bm_skm bmskm;

#endif
