// kmpskm.cc -- The -*- C++ -*- Knuth Morris Pratt skm methods.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#include "kmpskm.h"
#include "util.h"    // itos()
#include <iomanip.h> // setw()

/**

  KMP_SKM

**/

// PREPROCESSOR & MATCHER
#define KMPSKM_CC

#define  PREPROCESSOR preprocess        // Preprocess function
#define  MATCHER      match             // Match Function
#include "kmpskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER

#define  KMPSKM_RUN
#define  PREPROCESSOR run_preprocessor  // Run Preprocessor
#define  MATCHER      run_matcher       // Run Matcher
#include "kmpskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER
#undef   KMPSKM_RUN            

#undef KMPSKM_CC            

// print preprocessed data
ostream & kmp_skm::print_pdata (ostream &o) const {

  skm_patt::size_type i;
  skm_patt::iterator  pi;

  // PAT POSITION
  o << "i         : ";
  for (i=0; i < pat_size ; i++)
    o << setw(3) << i;
  o << "\n";

  // PAT
  pi = pat_beg;
  o << "pat [i]   : ";
  for (i=0; i < pat_size ; i++)
    o << "  " << *pi++;
  o << "\n";

  // SHIFT
  o << "shift [i] : ";
  for (i=0; i < pat_size; i++)
    o << setw(3) << shift[i];
  o << "\n";

  // RESUME
  o << "resume    : " << resume << "\n";
  return o;
};

// DISPLAY ANALYSIS
void kmp_skm::algorithm_step(breakpoint WHERE, int count) {

  adev_mpatt mp(pat);

  switch (WHERE) {

  case INIT_PREPROCESSOR:
    ad->message("PREPROCESSING ...");
    
    mp.update_pos(ppi - pat_beg);
    ad->matched_patt(mp);
    {
      string s = "shift ["; s += itos(ppi - pat_beg); s += "]";
      ad->diff_pdata(s, *shi);
    } 
    ad->diff_pdata("resume ", resume);
    break;

  case PREPROCESSOR_STEP:
    
    mp.update_pos(ppi - pat_beg);
    ad->matched_patt(mp);
    {
      string s = "shift ["; s += itos(ppi - pat_beg); s += "]";
      ad->diff_pdata(s, *shi);
    }
    ad->diff_pdata("resume ", resume);
    break;

  case INIT_MATCHER:
    ad->message("PREPROCESSOR STRUCTURES");
    ad->patt_diff_pdata("shift ", pat, shift);
    ad->diff_pdata("resume", resume);
    ad->message("SEARCHING ...");
    ad->matched_text(txt, ti, mp);
    break;

  case MATCHER_STEP:
    if(ti < txt_end - pat_size)
      ad->matched_text(txt, ti, mp);
    break;

  case PARTIAL_MATCH:
    ad->message("-- PARTIAL MATCH --");
    break;

  case PARTIAL_MATCH_STEP:
    if(ti < txt_end - pat_size) {
      mp.update_pos(pmpi - pat_beg);
      mp.matched_symbols(0, pmpi - pat_beg);
      ad->matched_text(txt, ti - (pmpi - pat_beg), mp);
      if (*ti != *pmpi) {
        string s = "shift ["; s += itos(pmpi - pat_beg); s += "]";
        ad->diff_pdata(s, shift[pmpi - pat_beg]);
      }
    }
    break;

  case FULL_MATCH: 
    mp.matched_symbols(0, pat_size);
    ad->matched_text(txt, ti - (pmpi - pat_beg), mp);
    ad->message("-- FULL MATCH --");
    break;

  case MATCH_RECOVERY:
    ad->diff_pdata("resume ", resume);
    break;
  };
};

// STATIC OBJECTS
kmp_skm kmpskm;
