// kmpskm.cpp -- The -*- C++ -*- Knuth Morris Pratt skm preprocessor & matcher.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifdef KMPSKM_CC

// Knuth Morris Pratt preprocessor
void kmp_skm::PREPROCESSOR () {

  // clear shift vector
  shift.erase(shift.begin(), shift.end());  // erase values
  shift.reserve(pat_size);                  // reserve space

  // allocate shift
  shift.insert(shift.begin(), pat_size + 1, 0);

  // init shift vector & resume
  shi    = shift.begin();
  resume = -1;
  ppi    = pat_beg;

  *shi = resume; // shift[0]=-1

#ifdef KMPSKM_RUN
  report_step(INIT_PREPROCESSOR); // Step 0
  if (stop) return;
#endif

  while (ppi != pat_end) {

    if (resume < 0 || *ppi == *(pat_beg + resume)) {

      shi++; ppi++; resume++;

      if ( (ppi != pat_end) && (*ppi == *(pat_beg + resume)) ) {
	*shi = shift[resume];
      } else {
	*shi = resume;
      }
    } else {
      resume = shift[resume];
    }

#ifdef KMPSKM_RUN
    report_step(PREPROCESSOR_STEP); // update values or resume
    if (stop) return;
#endif

  } // end while

};

// Knuth Morris Pratt matcher
astring::size_type kmp_skm::MATCHER (){
  // LOCAL OPTIMIZATIONS

  // USE p0 INSTEAD OF *pat_beg 
  register skm_patt::value_type p0 = *pat_beg;  

  // USE ti_last INSTEAD OF txt_end - ti ON while LOOP
  register astring::iterator   ti_last = txt_end - pat_size;

#ifndef KMPSKM_RUN
  // Redefine the text iterator declaring it in register
  register astring::iterator   ti = this->ti;
#endif

#ifdef KMPSKM_RUN
  report_step(INIT_MATCHER); // Step 0, get first text positions
  if (stop) return txt_size;
#endif

  // while the text can give us a match
  // while (txt_end - ti >= pat_size) { 
  while (ti < ti_last) { 

    if (*ti == p0) {

#ifdef KMPSKM_RUN
      report_step(PARTIAL_MATCH);
      if (stop) return txt_size;
#endif

      pmpi=pat_beg;

      do {

	pmpi++; ti++;

	// if pattern matched apply match function
	if (pmpi == pat_end) {
	  
	  // Internal variables
	  astring::size_type midx; // match index 
	    
	  midx = (ti - pat_size) - txt_beg; // ti is after the pattern
	    
#ifdef KMPSKM_RUN
	  report_step(FULL_MATCH);
	  if (stop) return txt_size;
#endif
       	  if (report_match(midx)) {
	    return (midx);                   // ... return the match
	  } else {
#ifdef KMPSKM_RUN
	    report_step(MATCH_RECOVERY);
	    if (stop) return txt_size;
#endif
	    if (resume < 0) {       // advance text
	      ti++; 
#ifdef KMPSKM_RUN
	      report_step(MATCHER_STEP);
	      if (stop) return txt_size;
#endif
	      break;
	    } else if (resume == 0) {
#ifdef KMPSKM_RUN
	      report_step(MATCHER_STEP);
	      if (stop) return txt_size;
#endif
	      break;
	    };

	    pmpi = pat_beg + resume; 
	    
#ifdef KMPSKM_RUN
	    report_step(PARTIAL_MATCH);
	    if (stop) return txt_size;
#endif
	  } // end if
	  
	} // end if (pmpi == pat_end)

#ifdef KMPSKM_RUN
	report_step(PARTIAL_MATCH_STEP); // update values
	if (stop) return txt_size;
#endif
	
	if (*pmpi != *ti) {
	  
	  pmpsi = shift[pmpi - pat_beg];
	  
	  if (pmpsi < 0) {         // Advance text	  
	    ti++;
#ifdef KMPSKM_RUN
	    report_step(MATCHER_STEP);
	    if (stop) return txt_size;
#endif
	    break;
	  } else {
	    
	    pmpi = pat_beg + pmpsi;	
	    
#ifdef KMPSKM_RUN
	    report_step(PARTIAL_MATCH_STEP); // update values
	    if (stop) return txt_size;
#endif
	    
	    if (pmpi == pat_beg)      // Reset to begining of the pattern
	      break;
	  }
	} // end if (*pmpi != *ti)
	
      } while (*pmpi == *ti);
      
    } else {
      
      ti++;
      
#ifdef KMPSKM_RUN
      report_step(MATCHER_STEP); // advance text
      if (stop) return txt_size;
#endif

    } // end if (*ti == p0)

  } // end while
  
  return (txt_size);

};

#endif
