//  kmpskm.h -- The -*- C++ -*- Knuth Morris Pratt skm interface.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _KMPSKM_H_
#define _KMPSKM_H_

#include "skm.h"
#include <vector> // skip table


// =====================================================================
// KMP SKM (Knuth Morris Pratt Single Keyword Matcher) : Class Interface
// =====================================================================

class kmp_skm : public skm_algo {
public:

protected:

  // internal variables
  vector<astring::difference_type>           shift; // Shift Table
  vector<astring::difference_type>::iterator shi;   // shift table iterator
  skm_patt::iterator                         ppi;   // preprocess pattern iterator

  skm_patt::iterator        pmpi;     // partial match pattern iterator
  skm_patt::difference_type pmpsi;    // partial match pattern shift index
  skm_patt::difference_type resume;   // shift[pat_size], restart after match

public:

  // constructor / destructor
  kmp_skm() : skm_algo() {};
  kmp_skm(const alpha &a) : skm_algo(a) {};

  ~kmp_skm() {};

  // long algorithm name
  virtual 
  const char * name() const { return "Knuth Morris Pratt"; };

  // short algorithm name
  virtual 
  const char * id() const { return "kmp"; };

  // print preprocesed data 
  virtual 
  ostream & print_pdata(ostream &o) const;

protected:

  virtual 
  void preprocess ();                // preprocess the pattern

  virtual 
  void run_preprocessor();           // run the preprocessor

  virtual 
  astring::size_type match();       // match the pattern

  virtual
  astring::size_type run_matcher(); // run the matcher

  virtual
  void algorithm_step(breakpoint WHERE, int count);
  // algorithm step function
};

// STATIC OBJECTS
extern kmp_skm kmpskm;

#endif
