// krskm.cc -- The -*- C++ -*- Karp Rabin skm methods.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#include "krskm.h"
#include <iomanip.h> // setw()

/**

  KR_SKM

**/

// PREPROCESSOR & MATCHER
#define KRSKM_CC

#define  PREPROCESSOR preprocess        // Preprocess function
#define  MATCHER      match             // Match Function
#include "krskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER

#define  KRSKM_RUN
#define  PREPROCESSOR run_preprocessor  // Run Preprocessor
#define  MATCHER      run_matcher       // Run Matcher
#include "krskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER
#undef   KRSKM_RUN            

#undef KRSKM_CC            

// print preprocessed data
ostream & kr_skm::print_pdata (ostream &o) const {

  // PAT
  o << "pat             : " << *pat  << "\n";
  // Constants
  o << "Prime(Q)        : " << Q     << "\n";
  o << "Base(B)         : " << B     << "\n";
  o << "Base shift (BS) : " << BS    << "\n";
  o << "Bm              : " << Bm    << "\n";
  o << "hpat            : " << hpat  << "\n";

  return o;
};

// DISPLAY ANALYSIS
void kr_skm::algorithm_step(breakpoint WHERE, int count) {

  adev_mpatt mp(pat, pat_end - pat_beg);

  switch (WHERE) {

  case INIT_PREPROCESSOR:
    ad->message("PREPROCESSING ...");
    break;

  case PREPROCESSOR_STEP:

    mp.update_pos(pmpi - pat_beg);

    ad->matched_patt(mp);
    ad->int_pdata("Bm  ", Bm);
    ad->int_pdata("hpat", hpat);

    break;

  case INIT_MATCHER:
    ad->message("SEARCHING ...");
    ad->matched_text(txt, ti, mp);
    ad->int_pdata("htxt", htxt);
    ad->int_pdata("hpat", hpat);
    break;

  case MATCHER_STEP:
    if(ti < txt_end - pat_size) {
      ad->matched_text(txt, ti, mp);
      ad->int_pdata("htxt", htxt);
      ad->int_pdata("hpat", hpat);
    }
    break;

  case PARTIAL_MATCH:
    ad->message("-- PARTIAL MATCH --");
    break;

  case PARTIAL_MATCH_STEP:
    if(ti < txt_end - pat_size) {
      mp.update_pos(pmpi - pat_beg);
      mp.matched_symbols(0, pmpi - pat_beg);
      ad->matched_text(txt, ti, mp);
    }
    break;

  case FULL_MATCH: 
    mp.matched_symbols(0, pat_size);
    ad->matched_text(txt, ti, mp);
    ad->message("-- FULL MATCH --");
    break;

  case MATCH_RECOVERY:
    break;
  };
};

// STATIC OBJECTS
kr_skm krskm;
