// krskm.cpp -- The -*- C++ -*- Karp Rabin skm prep & matcher. 
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifdef KRSKM_CC

// Karp Rabin preprocessor
void kr_skm::PREPROCESSOR () {
    
  //internal variables
  skm_patt::size_type i;
  skm_patt::iterator  pi;

  // Set the BASE, the BASE SHIFT & the PRIME

  // Compute minimum BS that verifies: pow(2, BS) >= alpha_size
  BS = 1;
  while ((alpha_size >> BS) > 1)  // assuming alpha_size != 0
    BS++;
  if (alpha_size % BS != 0)       // If alpha_size is not a power of 2 ... 
    BS++;                         // ... get the next power of 2

  B = 1 << BS; // B == minimum power of 2 that verifies: B >= alpha_size

  Q = PRIMES[BS];

  // Start Preprocessing
  hpat = 0;
  Bm = 0;

#ifdef KRSKM_RUN
  report_step(INIT_PREPROCESSOR); // Step 0
  if (stop) return;
#endif

  // The first step is out of the while because the Bm formula assumes
  // Bm != 0
  pmpi=pat_beg;

  Bm   = 1;
  hpat = aindex(*pmpi); // There's no need to compute the modulus

#ifdef KRSKM_RUN
  report_step(PREPROCESSOR_STEP); // Update Bm, pmpi & hpat
  if (stop) return;
#endif

  pmpi++;

  while (pmpi != pat_end) {
    Bm = (Bm << BS) % Q;
    hpat  = ( (hpat << BS) + aindex(*pmpi) ) % Q;
#ifdef KRSKM_RUN
    report_step(PREPROCESSOR_STEP); // Update Bm, pmpi & hpat
    if (stop) return;
#endif
    pmpi++;
  }

};

// Karp Rabin matcher
astring::size_type kr_skm::MATCHER (){

  // LOCAL OPTIMIZATIONS

  // USE ti_last INSTEAD OF txt_end - ti ON while LOOP
  register astring::iterator ti_last = txt_end - pat_size;

#ifndef KRSKM_RUN
  // Redefine the internal pointers declaring them in registers
  register astring::iterator   ti = this->ti;

  register int hpat= this->hpat;
  register int htxt;
#endif

  // init htext
  htxt = 0;
  pmti = ti;
  pmpi = pat_beg;

  while (pmpi != pat_end) {

    htxt = ( (htxt << BS) + aindex(*pmti) ) % Q;
    pmti++; pmpi++;

  }

#ifdef KRSKM_RUN
  report_step(INIT_MATCHER); // Step 0, get first text positions
  if (stop) return txt_size;
  // This step is here and not before because the initial htxt can
  // be computed at the same time as the preprocessing ... maybe
  // it shouldn't be that way, but its useful for the display ... :-)
#endif
    
  // while the text can give us a match
  while (ti < ti_last) { 
    
    if (hpat==htxt) {
      // hash values are equal, compare symbols 

      pmti = ti;
      pmpi = pat_beg;
      
#ifdef KRSKM_RUN
      report_step(PARTIAL_MATCH); // Set pmpi & pmti
      if (stop) return txt_size;
#endif

      while (*pmpi == *pmti) {

#ifdef KRSKM_RUN
	report_step(PARTIAL_MATCH_STEP); // Update pmpi & pmti
	if (stop) return txt_size;
#endif

	pmpi++; pmti++;

	// If pattern matched apply match function
	if (pmpi == pat_end) {

	  // Internal variables
	  astring::size_type midx; // match index 

	  midx = ti - txt_beg;

#ifdef KRSKM_RUN
	  report_step(FULL_MATCH); // Report full match to the stepper
	  if (stop) return txt_size;
#endif
	    
	  if (report_match(midx)) {
	    return (midx);                   // ... return the match
	  } else {
#ifdef KRSKM_RUN
	    report_step(MATCH_RECOVERY);
	    if (stop) return txt_size;
#endif
	    break;                   // ... continue searching
	  }
	
	} // end if
      
      } // end while

    }
    
    htxt= ( htxt + (Q << BS) - ( aindex(*ti) * Bm ) ) % Q;
    htxt= ( (htxt << BS) + aindex(*(ti + pat_size)) ) % Q;

    ti++;
    
#ifdef KRSKM_RUN
    report_step(MATCHER_STEP); // advance text & hash value
    if (stop) return txt_size;
#endif
  }

  return (txt_size);

};

#endif
