/*  skmacdev.cc -- The -*- C++ -*- skm analisys curses device methods
    Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
    This file is part of the C++ Alphabetic Data Management Library. */

#include "skmcadev.h"

#if HAVE_CURSES_H

#include <string.h>   // strlen
#include <iostream.h>
#include "atoken.h"
#include "util.h"     // itos

// ========================
// Constructor / Destructor
// ========================

skm_cadev::skm_cadev (NCursesWindow *w) : mainw (w) {

  // Text Area
  textw = new NCursesWindow(*mainw, 7, mainw->width(), 0, 0, 'r');
  // Mesg Area
  mesgw = new NCursesWindow(*mainw, 1, mainw->width(), 7, 0, 'r');
  // Input Output Area
  iow   = new NCursesWindow(*mainw, 1, mainw->width(), 8, 0, 'r');
  // Prep Area
  prepw = new NCursesWindow(*mainw, mainw->height()-9, mainw->width(),
			    9, 0, 'r');
}

skm_cadev::~skm_cadev (){
  delete prepw;
  delete iow;
  delete mesgw;
  delete textw;
}

// =====
// MATCH
// =====

void skm_cadev::matched_patt (const adev_mpatt &mp){
  skm_patt::iterator  pi;
  skm_patt::size_type i, psz;
  
  psz = (mp.pattern())->size();
  pi  = (mp.pattern())->begin();
  
  for (i = 0; i < psz; i++)
    textw->addch(0,i,*pi++);
  
  textw->addch('\n'); 
  
  for (i = 0; i < psz; i++) {
    if (mp.matched(i))      textw->addch ('*');
    else if (mp.current(i)) textw->addch ('-');
    else                    textw->addch (' ');
  }

  textw->refresh();
};

#include <list>

void skm_cadev::matched_text (astring *txt, astring::iterator ppit,
			      const adev_mpatt &mp) {

  astring::iterator          tlbeg, tlend;
  skm_patt::iterator         pbeg,  pend;
  skm_patt::size_type        psz;

  astring::iterator          ti, tj;
  skm_patt::iterator         pi;
  skm_patt::size_type        i, plbeg, plend;

  vector<astring::iterator> winlines;

  adtoken line("line","\n", "\n");

  //
  // Setup parameters
  //

  // Pattern
  pbeg = (mp.pattern())->begin();
  pend = (mp.pattern())->end();
  psz  = (mp.pattern())->size();

  // Text LINE
  tlbeg = line.begin(*txt, ppit);
  tlend = line.end(*txt, ppit + psz - 1);

  // First we jump to the first screen line (there is no '\n' between
  // tlbeg and ppit, so discard the first ((ppit - tlbeg)/mchpl) lines
  // of size mchpl.

  ti = tlbeg + (((ppit - tlbeg) / textw->width()) * textw->width());
  winlines.push_back(ti);

  // Get Text Lines that contain the pattern (vector of begin and end
  // of each line -- note that the end of one line is the begin of the
  // next one)

  while (ti < ppit + psz) {
    i=0;
    while (ti != tlend && *ti != '\n') {
      if (i == textw->width() - 1)
        break;
      ti++; i++;
    }
    if (ti != tlend && *ti == '\n')
      ti++;
    winlines.push_back(ti);
    if (ti == tlend)
      break;
  } 

  // print the lines that contain the pattern
  int nl = 0;
  ti = winlines[nl];
  nl++;

  while (nl < winlines.size()) {

    // get the first pattern position to print
    if (ti > ppit)
      plbeg = ti - ppit;
    else
      plbeg = 0;
    
    // get the last pattern position
    if (winlines[nl] < ppit + psz)
      plend = winlines[nl] - ppit;
    else
      plend = psz;

    //
    // Print pattern over text
    //

    // If we're on the 1st line, print blanks
    if (plbeg == 0) {
      for (i=0; i < ppit - ti; i++)
        textw->addch(' ');
    }
    
    // Actual printing
    for (i = plbeg; i != plend; i++)
      textw->addch(*(pbeg + i));

    textw->addch('\n'); 

    //
    // Print text line
    //

    for (tj=ti; tj != winlines[nl]; tj++)
      textw->addch(*tj);

    if (*(tj-1) != '\n') 
      textw->addch('\n'); 

    //
    // Print pattern marks under the text
    //

    // If we're on the 1st line, print blanks
    if (plbeg == 0) {
      for (i=0; i < ppit - ti; i++)
        textw->addch(' ');
    }

    // Actual printing
    for (i=plbeg; i != plend; i++) {
      if (mp.matched(i))      textw->addch ('*');
      else if (mp.current(i)) textw->addch ('-');
      else                    textw->addch (' ');
    }

    textw->addch('\n'); 

    // GOTO next line
    ti = winlines[nl];
    nl++;
  }

  textw->refresh();
};

// =====
// PDATA
// =====

// -------------
// Single values
// -------------

void skm_cadev::int_pdata (const string & name, int value){
  string m = name;
  m += " = ";
  m += itos (value); 

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::size_pdata (const string & name, astring::size_type value){
  string m = name;
  m += " = ";
  m += ltos (value); 

  prepw->addnstr(m.begin(), m.size()); 
  prepw->addch('\n');
  prepw->refresh();
};


void skm_cadev::diff_pdata (const string & name,
			   astring::difference_type value) {
  string m = name;
  m += " = ";
  m += ltos (value); 

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::str_pdata (const string & name, const string & value) {

  string m = name;
  m += " = ";
  m += value ; 

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

// ------------
// Patt Vectors
// ------------

void skm_cadev::patt_size_pdata(const string & name, skm_patt * pat,
				const vector<astring::size_type> &v) {

  string              m;
  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  pi       = pat->begin();
  m        = "";

  // PAT POSITION
  for (i=0; i < pat_size ; i++) {
    string n = ltos(i);
    switch (n.size()) {
    case  1: m += "  "; break;
    case  2: m += " "; break;
    default: break;
    }
    m += n;
  }
  m += " : i\n";

  // PAT
  for (i=0; i < pat_size ; i++) {
    m += "  "; m += *pi++;
  }
  m += " : pat\n";

  // VECTOR
  for (i=0; i < pat_size; i++) {
    string n = ltos(v[i]);
    switch (n.size()) {
    case  1: m += "  "; break;
    case  2: m += " "; break;
    default: break;
    }
    m += n;
  }
  m += " : "; m += name; m += "[i]\n";
  
  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::patt_diff_pdata(const string & name, skm_patt * pat,
				const vector<astring::difference_type> &v) {

  string              m;
  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  pi       = pat->begin();
  m        = "";

  // PAT POSITION
  for (i=0; i < pat_size ; i++) {
    string n = ltos(i);
    switch (n.size()) {
    case  1: m += "  "; break;
    case  2: m += " "; break;
    default: break;
    }
    m += n;
  }
  m += " : i\n";

  // PAT
  for (i=0; i < pat_size ; i++) {
    m += "  "; m += *pi++;
  }
  m += " : pat\n";
  
  // VECTOR
  for (i=0; i < pat_size; i++) {
    string n = ltos(v[i]);
    switch (n.size()) {
    case  1: m += "  "; break;
    case  2: m += " "; break;
    default: break;
    }
    m += n;
  }
  m += " : "; m += name; m += "[i]\n";
  
  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::patt_str_pdata(const string & name, skm_patt * pat,
			       const vector<string> &v) {

  string              m;
  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  m        = "";

  m = " i pat[i] "; m += name; m += "[i]\n\n";

  for (i=0; i < pat_size ; i++) {
    string n;

    // PAT POSITION
    n = ltos(i);
    switch (n.size()) {
    case  1: m += "  "; break;
    case  2: m += " "; break;
    default: break;
    }
    m += n;
    
    // PAT
    m += "       ";
    m += *pi++;
    // VECTOR
    m += " ";
    m += v[i];

    m += "\n";
  }

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

// -------------
// Alpha Vectors
// -------------

void skm_cadev::alpha_size_pdata(const string & name, alpha * al,
				 const vector<astring::size_type> &v,
				 astring::size_type def) {
  string m;
  int    i, alpha_size;

  m          = "";
  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (v[i] != def) {
      m += name; m += "[\'"; m += itos (al->value(i)); m += "\'] :";
      m += ltos (v[i]);
      m += '\t';
    }
  }

  // DEFAULT VALUE
  m += name; m += "[ * ]"; m += ltos (def); m += '\n';

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::alpha_diff_pdata(const string & name, alpha * al,
				 const vector<astring::difference_type> &v,
				 astring::difference_type def) {
  string m;
  int    i, alpha_size;

  m          = "";
  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (v[i] != def) {
      m += name; m += "[\'"; m += itos (al->value(i)); m += "\'] :";
      m += ltos (v[i]);
      m += '\t';
    }
  }

  // DEFAULT VALUE
  m += name; m += "[ * ]"; m += ltos (def); m += '\n';

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

void skm_cadev::alpha_str_pdata(const string & name, alpha * al,
				const vector<string> &v, string def) {
  string m;
  int    i, alpha_size;

  m          = "";
  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (! (v[i] == def)) {
      m += name; m += "[\'"; m += itos (al->value(i)); m += "\'] :";
      m += v[i];
      m += '\t';
    }
  }

  // DEFAULT VALUE
  m += name; m += "[ * ]"; m += def;  m += '\n';

  prepw->addnstr(m.begin(), m.size());
  prepw->addch('\n');
  prepw->refresh();
};

// ========
// Messages
// ========

void skm_cadev::message(const string & msg) {
  mesgw->erase();
  mesgw->addnstr(msg.begin(), msg.size());
};

// ================
// User Interaction
// ================

bool skm_cadev::user_io (breakpoint i) {

  bool res = false;

  // Refresh Messages
  mesgw->refresh();
  
  napms(msv[i]);                      // wait ms miliseconds

  if (gchv[i]) {
    string m = gchv[i]->getmsg();
    string::iterator mi = m.begin();

    iow->erase();
    iow->addnstr(m.begin(), m.size());
    iow->refresh();

    res = (*gchv[i])(iow->getch());

    iow->erase(); 
    iow->refresh();
  }

  // Erase all windows
  textw->erase();
  mesgw->erase();
  prepw->erase();

  return res;  
}

#endif
