// skmcadev.h -- The -*- C++ -*- skm curses analisys device interface
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _SKMCDEV_H_
#define _SKMCDEV_H_

#include "admlconf.h"

#if HAVE_CURSES_H

#include "cursesw.h"
#include "skmadev.h"

// ====================================================
// Curses ADEV (Text analysis device) : Class Interface
// ====================================================

class skm_cadev : public skm_adev {

protected:

  NCursesWindow *mainw; // Main Window

  NCursesWindow *textw; // Text Area
  NCursesWindow *iow;   // Input/Output Window
  NCursesWindow *mesgw; // Messages Window
  NCursesWindow *prepw; // Preprocessed Data Area

public:

  // typedefs
  typedef skm_alpha      alpha;
  typedef skm_breakpoint breakpoint;

  // ========================
  // Constructor / Destructor
  // ========================

  skm_cadev (NCursesWindow *w); 

  ~skm_cadev ();
  
  // =====
  // MATCH
  // =====

  virtual
  void matched_patt (const adev_mpatt &mp);

  virtual
  void matched_text (astring *txt, astring::iterator ppit,
                     const adev_mpatt &mp);

  // =====
  // PDATA
  // =====

  // Single values
  virtual
  void int_pdata (const string & name, int value);

  virtual
  void size_pdata (const string & name, astring::size_type value);

  virtual
  void diff_pdata (const string & name, astring::difference_type value);

  virtual
  void str_pdata (const string & name, const string & value);

  // Patt Vectors
  virtual
  void patt_size_pdata(const string & name, skm_patt * pat,
                       const vector<astring::size_type> &v);
  virtual
  void patt_diff_pdata(const string & name, skm_patt * pat,
                       const vector<astring::difference_type> &v);
  virtual
  void patt_str_pdata(const string & name, skm_patt * pat,
                      const vector<string> &v);

  // Alpha Vectors
  virtual
  void alpha_size_pdata(const string & name, alpha * al,
			const vector<astring::size_type> &v,
			astring::size_type def);
  virtual
  void alpha_diff_pdata(const string & name, alpha * al,
			const vector<astring::difference_type> &v,
			astring::difference_type def);
  virtual
  void alpha_str_pdata(const string & name, alpha * al,
		       const vector<string> &v, string def);

  // ========
  // Messages
  // ========

  virtual
  void message(const string & msg); // Print a message 

  // ================
  // User interaction
  // ================

  virtual
  bool user_io (breakpoint i); // continue running ?

};

#endif

#endif
