/*  skmtadev.cc -- The -*- C++ -*- skm text analisys device methods
    Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
    This file is part of the C++ Alphabetic Data Management Library. */

#include <iostream.h>
#include <string.h>   // strlen
#include "skmtadev.h"
#include "atoken.h"

// ========================
// Constructor / Destructor
// ========================

skm_tadev::skm_tadev (int max_chars_per_line) : mchpl (max_chars_per_line) {};

skm_tadev::~skm_tadev () {};

// =====
// MATCH
// =====

void skm_tadev::matched_patt (const adev_mpatt &mp){

  skm_patt::iterator  pi;
  skm_patt::size_type i, psz;

  psz = (mp.pattern())->size();
  pi  = (mp.pattern())->begin();

  for (i = 0; i < psz; i++)
    cout << *pi++;
  cout << "\n";

  for (i=0; i < psz; i++) {
    if (mp.matched(i))      cout << "*";
    else if (mp.current(i)) cout << "-";
    else                    cout << " ";
  }

  cout << "\n" << endl;
};

#include <vector>

void skm_tadev::matched_text (astring *txt, astring::iterator ppit,
			      const adev_mpatt &mp) {

  astring::iterator         tlbeg, tlend;
  skm_patt::iterator        pbeg,  pend;
  skm_patt::size_type       psz;

  astring::iterator         ti, tj;
  skm_patt::iterator        pi;
  skm_patt::size_type       i, plbeg, plend;

  vector<astring::iterator> txtlines;

  adtoken line("line","\n", "\n");

  //
  // Setup parameters
  //

  // Pattern
  pbeg = (mp.pattern())->begin();
  pend = (mp.pattern())->end();
  psz  = (mp.pattern())->size();

  // Text LINE
  tlbeg = line.begin(*txt, ppit);
  tlend = line.end(*txt, ppit + psz - 1);

  // First we jump to the first print line (there is no '\n' between
  // tlbeg and ppit, so discard the first ((ppit - tlbeg)/mchpl) lines
  // of size mchpl.

  ti = tlbeg + ( ( (ppit - tlbeg) / mchpl) * mchpl);
  txtlines.push_back(ti);

  // Get Text Lines that contain the pattern (vector of begin and end
  // of each line -- note that the end of one line is the begin of the
  // next one)

  while (ti < ppit + psz) {
    i=0;
    while (ti != tlend && *ti != '\n') {
      if (i == mchpl)
        break;
      ti++; i++;
    }
    if (ti != tlend && *ti == '\n')
      ti++;
    txtlines.push_back(ti);
    if (ti == tlend)
      break;
  } 

  // print the lines that contain the pattern
  int nl=0;
  ti = txtlines[nl];
  nl++;

  while(nl < txtlines.size()) {

    // get the first pattern position to print
    if (ti > ppit)
      plbeg = ti - ppit;
    else
      plbeg = 0;
    
    // get the last pattern position
    if (txtlines[nl] < ppit + psz)
      plend = txtlines[nl] - ppit;
    else
      plend = psz;

    //
    // Print pattern over text
    //

    // If we're on the 1st line, print blanks
    if (plbeg == 0) {
      for (i=0; i < ppit - ti; i++)
        cout << " ";
    }
    
    // Actual printing
    for (i = plbeg; i != plend; i++)
      cout << *(pbeg + i);

    cout << "\n";

    //
    // Print text line
    //

    for (tj=ti; tj != txtlines[nl]; tj++)
      cout << *tj;

    if (*(tj-1) != '\n') 
      cout << "\n"; 

    //
    // Print pattern marks under the text
    //

    // If we're on the 1st line, print blanks
    if (plbeg == 0) {
      for (i=0; i < ppit - ti; i++)
        cout << " ";
    }

    // Actual printing
    for (i=plbeg; i != plend; i++) {
      if (mp.matched(i))      cout << "*";
      else if (mp.current(i)) cout << "-";
      else                    cout << " ";
    }

    cout << "\n"; 

    // GOTO next line
    ti = txtlines[nl];
    nl++;
  }

  cout << endl;
};

// =====
// PDATA
// =====

// -------------
// Single values
// -------------

void skm_tadev::int_pdata (const string & name, int value){
  cout << name << " = " << value << "\n" << endl;
};

void skm_tadev::size_pdata (const string & name, astring::size_type value){
  cout << name << " = " << value << "\n" << endl;
};


void skm_tadev::diff_pdata (const string & name,
			   astring::difference_type value) {
  cout << name << " = " << value << "\n" << endl;
};

void skm_tadev::str_pdata (const string & name, const string & value) {
  cout << name << " = " << value << "\n" << endl;
};

// ------------
// Patt Vectors
// ------------

void skm_tadev::patt_size_pdata(const string & name, skm_patt * pat,
				const vector<astring::size_type> &v) {

  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  pi = pat->begin();

  // PAT POSITION
  for (i=0; i < pat_size ; i++)
    cout << setw(3) << i;
  cout << "  : i\n";

  // PAT
  for (i=0; i < pat_size ; i++)
    cout << "  " << *pi++;
  cout << "  : pat[i]\n";

  // VECTOR
  for (i=0; i < pat_size; i++)
    cout << setw(3) << v[i];
  cout << " : " << name << "[i]\n";
  cout << endl;
};

void skm_tadev::patt_diff_pdata(const string & name, skm_patt * pat,
				const vector<astring::difference_type> &v) {

  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  pi = pat->begin();

  // PAT POSITION
  for (i=0; i < pat_size ; i++)
    cout << setw(3) << i;
  cout << "  : i\n";

  // PAT
  for (i=0; i < pat_size ; i++)
    cout << "  " << *pi++;
  cout << "  : pat[i]\n";

  // VECTOR
  for (i=0; i < pat_size; i++)
    cout << "  "  << v[i];
  cout << " : " << name << "[i]\n";
  cout << "\n" << endl;
};

void skm_tadev::patt_str_pdata(const string & name, skm_patt * pat,
                               const vector<string> &v) {

  skm_patt::size_type i, pat_size;
  skm_patt::iterator  pi;

  pat_size = pat->size();
  pi = pat->begin();

  cout << "  i  pat[i] " << name << "[i]\n\n";

  for (i=0; i < pat_size ; i++)
    cout << setw(3) << i << "       " << *pi++ << " " << v[i] << "\n";

  cout << "\n" << endl;
};

// -------------
// Alpha Vectors
// -------------

void skm_tadev::alpha_size_pdata(const string & name, alpha * al,
				const vector<astring::size_type> &v,
				astring::size_type def) {

  int i, alpha_size;

  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (! (v[i] == def)) {
      cout << name << "[\'" << al->value(i) << "\'] :"
	   << setw(3) << v[i] << "\n";
    }
  }

  // DEFAULT VALUE
  cout << name << "[ * ] :" << setw(3) << def << "\n";
  cout << "\n" << endl;
};

void skm_tadev::alpha_diff_pdata(const string & name, alpha * al,
				 const vector<astring::difference_type> &v,
				 astring::difference_type def) {

  int i, alpha_size;

  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (v[i] != def) {
      cout << name << "[\'" << al->value(i) << "\'] :"
	   << setw(3) << v[i] << "\n";
    }
  }

  // DEFAULT VALUE
  cout << name << "[ * ] :" << setw(3) << def << "\n";
  cout << endl;
};

void skm_tadev::alpha_str_pdata(const string & name, alpha * al,
				const vector<string> &v, string def) {

  int i, alpha_size;

  alpha_size = al->size();

  // VECTOR
  for (i=0; i < alpha_size; i++){
    if (!(v[i] == def)) {
      cout << name << "[\'" << al->value(i) << "\'] :"
	   << setw(3) << v[i] << "\n";
    }
  }

  // DEFAULT VALUE
  cout << name << "[ * ] :" << setw(3) << def << "\n";
  cout << "\n" << endl;
};

// ========
// Messages
// ========

void skm_tadev::message(const string & msg) {
  cout << msg << "\n" << endl;
};

// ================
// User Interaction
// ================

#ifdef unix

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

bool skm_tadev::user_io (breakpoint i) {
  struct timeval timeout;
  char ch;

  if (msv[i] != 0) {
    timeout.tv_sec  =  msv[i] / 1000;
    timeout.tv_usec = (msv[i] % 1000) * 1000;
    select (0, NULL, NULL, NULL, &timeout);
  }

  if (gchv[i] != NULL) {
    cout << gchv[i]->getmsg();
    cin >> ch;
    cout << endl;
    return (*gchv[i])(ch);
  }

  return false;
}

#else

#include <unistd.h> // sleep(), a unistd.h header should exist (in DJGPP 
		    // and CW10 it does)

bool skm_tadev::user_io (breakpoint i) {

  if (msv[i] != 0) { sleep (msv[i] / 1000); };

  if (gchv[i] != NULL) {
    cout << gchv[i]->getmsg();
    cin >> ch;
    return *(gchf[i])(ch);
  }

  return false;
}

#endif
