// skmtadev.h -- The -*- C++ -*- analisys devices interface
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _SKMTADEV_H_
#define _SKMTADEV_H_

#include "skmadev.h"

// ==================================================
// SKM TADEV (Text analysis device) : Class Interface
// ==================================================

class skm_tadev : public skm_adev {

protected:

  int  mchpl;  // max chars per line

public:

  // typedefs
  typedef skm_alpha      alpha;
  typedef skm_breakpoint breakpoint;

  // ========================
  // Constructor / Destructor
  // ========================

  skm_tadev (int max_chars_per_line = 78);

  ~skm_tadev ();

  // =====
  // MATCH
  // =====

  virtual
  void matched_patt (const adev_mpatt &mp);

  virtual
  void matched_text (astring *txt, astring::iterator ppit,
                     const adev_mpatt &mp);

  // =====
  // PDATA
  // =====

  // Single values
  virtual
  void int_pdata (const string & name, int value);

  virtual
  void size_pdata (const string & name, astring::size_type value);

  virtual
  void diff_pdata (const string & name, astring::difference_type value);

  virtual
  void str_pdata (const string & name, const string & value);

  // Patt Vectors
  virtual
  void patt_size_pdata(const string & name, skm_patt * pat,
                       const vector<astring::size_type> &v);
  virtual
  void patt_diff_pdata(const string & name, skm_patt * pat,
                       const vector<astring::difference_type> &v);
  virtual
  void patt_str_pdata(const string & name, skm_patt * pat,
		      const vector<string> &v);

  // Alpha Vectors
  virtual
  void alpha_size_pdata(const string & name, alpha * al,
			const vector<astring::size_type> &v,
			astring::size_type def);
  virtual
  void alpha_diff_pdata(const string & name, alpha * al,
			const vector<astring::difference_type> &v,
			astring::difference_type def);
  virtual
  void alpha_str_pdata(const string & name, alpha * al,
		       const vector<string> &v, string def);

  // ========
  // Messages
  // ========

  virtual
  void message(const string & msg); // Print a message 

  // ================
  // User interaction
  // ================

  virtual
  bool user_io (breakpoint i); // continue running ?

};

#endif
