// soskm.cc -- The -*- C++ -*- Karp Rabin skm methods.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#include "soskm.h"
#include <iomanip.h> // setw()

/**

  SO_SKM

**/

// PREPROCESSOR & MATCHER
#define SOSKM_CC

#define  PREPROCESSOR preprocess        // Preprocess function
#define  MATCHER      match             // Match Function
#include "soskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER

#define  SOSKM_RUN
#define  PREPROCESSOR run_preprocessor  // Run Preprocessor
#define  MATCHER      run_matcher       // Run Matcher
#include "soskm.cpp"
#undef   PREPROCESSOR
#undef   MATCHER
#undef   SOSKM_RUN            

#undef SOSKM_CC            

// print preprocessed data
ostream & so_skm::print_pdata (ostream &o) const {

  alpha::size_type i;

  // PAT
  o << "pat        : " << *pat << "\n";

  // MASK  
  for (i=0; i != mask.size(); i++){
    if (mask[i] != ~0) {
      o << "mask [\'" << al.value(i) << "\'] : " << bits(mask[i]) << "\n";
    }
  }

  o << "mask [ * ] : " << bits(~0) << "\n";

  return o;
};

// DISPLAY ANALYSIS
void so_skm::algorithm_step(breakpoint WHERE, int count) {

  int        st, ms;
  adev_mpatt mp(pat);

  switch (WHERE) {

  case INIT_PREPROCESSOR:
    ad->message("PREPROCESSING ...");
    ad->str_pdata("mask [ * ]", string(bits(mask[0])));
    break;

  case PREPROCESSOR_STEP:
    if (count == 1) {
      mp.update_pos(pmpi - pat_beg);
      ad->matched_patt(mp);
      string s ("mask [\'"); s += *pmpi; s += "\']";
      ad->str_pdata(s, string(bits(mask[aindex(*pmpi)])));
    }
    break;

  case INIT_MATCHER:
    ad->message("PREPROCESSOR STRUCTURE");

    {
      vector<string> bitsv (mask.size());

      for (int i=0; i<mask.size(); i++)
        bitsv[i] = string(bits(mask[i]));

      ad->alpha_str_pdata("mask ", &al, bitsv, string(bits(~0)));
    }

    ad->message("SEARCHING ...");
    ad->matched_text(txt, ti, mp);
    break;

  case MATCHER_STEP:
    if(ti < txt_end - pat_size)
      ad->matched_text(txt, ti, mp);
    break;

  case PARTIAL_MATCH:
    ad->message("-- PARTIAL MATCH --");
    break;

  case PARTIAL_MATCH_STEP:
    if(ti < txt_end - pat_size) {
      st = state; 
      if (st != ~0) {
	ms=0;
	while (st != ~0) { st >>= 1; ms++; };
	
	mp.update_pos(ms);
	mp.matched_symbols(0, ms);
	ad->matched_text(txt, ti - ms, mp);
	ad->str_pdata("state     ", string (bits (state)));
	string s ("mask [\'"); s += *ti; s += "\']";
	ad->str_pdata(s, string(bits(mask[aindex(*ti)])));
	ad->str_pdata("state << 1", string (bits (state << 1)));
	ad->str_pdata("nextstate ", 
		      string (bits( (state << 1) | mask[aindex(*ti)] )));
      } else {
	ad->matched_text(txt, ti, mp);
      }
    }
    break;

  case FULL_MATCH:
    mp.update_pos(pat_size);
    mp.matched_symbols(0, pat_size);
    ad->matched_text(txt, pmti - pat_size, mp);
    ad->message("-- FULL MATCH --");
    break;

  case MATCH_RECOVERY:
    ad->str_pdata("state     ", string (bits (state)));    
    break;
  };
};

// STATIC OBJECTS
so_skm soskm;
