// soskm.cpp -- The -*- C++ -*- Shift Or skm prep & matcher. 
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifdef SOSKM_CC

// Shift Or preprocessor
void so_skm::PREPROCESSOR () {

  // internal variables
  unsigned int i; 

  // INIT MASK TABLE
  // clear mask vector
  mask.erase(mask.begin(), mask.end());            // erase values
  mask.reserve(alpha_size);                        // inform of space needed

  // allocate mask vector with assignment (mask[i] = ~0)
  mask.insert(mask.begin(), alpha_size, ~0);

  // init variables
  max_states = min(pat_size, sizeof(unsigned int) * 8);

  // compute the mask & the last state
  last_state = 1;
  pmpi       = pat_beg;

#ifdef SOSKM_RUN
  report_step(INIT_PREPROCESSOR); // step 0
  if (stop) return;

  report_step(PREPROCESSOR_STEP, alpha_size);
  // Inicialization of mask -> no psteps, but they are there
  if (stop) return;
#endif

  for (i=0; i < max_states; i++) {
    mask[ aindex(*pmpi) ] &= ~last_state;
#ifdef SOSKM_RUN
    report_step(PREPROCESSOR_STEP); // mask updates 
    if (stop) return;
#endif
    last_state <<= 1;
    pmpi++;
  }

  last_state = 1 << (max_states - 1);
}

// Shift Or matcher
astring::size_type so_skm::MATCHER (){

  // LOCAL OPTIMIZATIONS

  // USE p0 INSTEAD OF *pat_beg 
  register skm_patt::value_type p0 = *pat_beg;  

  // USE ti_last INSTEAD OF txt_end - ti ON while LOOP
  register astring::iterator   ti_last = txt_end - pat_size;

#ifndef SOSKM_RUN
  // Redefine the text iterator declaring it in register
  register astring::iterator   ti = this->ti;
#endif

#ifdef SOSKM_RUN
  report_step(INIT_MATCHER); // Step 0, get first text positions
  if (stop) return txt_size;
#endif

  // while the text can give us a match
  //  while (txt_end - ti >= pat_size) { 
  while (ti < ti_last) { 
 
    if (*ti == p0) {

    /* NOTE:

       To implement the extended versions of the shift_or algorithm we should
       use the following patial match test:

       if ((mask[ aindex(*ti) ] & 1) == 0) 

       That is, if the mask for the given symbol has a 0 in the first
       position we have a partial match.

       If we use the current test we can't use a class of symbols at
       the first pattern position.  */

      state = ~0;

#ifdef SOSKM_RUN
      report_step(PARTIAL_MATCH);
      if (stop) return txt_size;
#endif

      state = (state << 1) | (mask[ aindex(*ti) ]);

      while (state != ~0) {

	ti++;
		
	if ((state & last_state) == 0) {
	  
	  pmti = ti;
	  pmpi = pat_beg + max_states;
	  
	  if (pat_size > max_states) {  // if patt is bigger than mask bits ...
	    
	    while (*pmti == *pmpi) {    // ... sequential search

	      pmpi++; pmti++;
	      
#ifdef SOSKM_RUN
	      report_step(PARTIAL_MATCH_STEP); // update pmpi, pmti
	      if (stop) return txt_size;
#endif
	      
	      if (pmpi == pat_end) break;
	    } 
	    
	  } // endif
	    
	  // If the pattern matched, apply match function
	  if (pmpi == pat_end) {
	    
	    // Internal variables
	    astring::size_type midx; // match index 
	    
	    midx = (pmti - pat_size) - txt_beg;
	    
#ifdef SOSKM_RUN
	    report_step(FULL_MATCH);
	    if (stop) return txt_size;
#endif

	    if (report_match(midx)) {
	      return (midx);                   // ... return the match
	    } else {

	      state = state | last_state;      // ... erase match

#ifdef SOSKM_RUN
	      report_step(MATCH_RECOVERY);
	      if (stop) return txt_size;
#endif

	      if (state == ~0) {
#ifdef SOSKM_RUN4
		report_step(MATCHER_STEP); // update ti
		if (stop) return txt_size;
#endif
		break;
	      }
#ifdef SOSKM_RUN
	      report_step(PARTIAL_MATCH);
	      if (stop) return txt_size;
#endif	      
	    }
	    
	  } // end if (pmpi == pat_end)

	} // end if

#ifdef SOSKM_RUN
	report_step(PARTIAL_MATCH_STEP); // update ti
	if (stop) return txt_size;
#endif

	state = (state << 1) | (mask[ aindex(*ti) ]);

      } // end while

    } else {
      ti++;
#ifdef SOSKM_RUN
      report_step(MATCHER_STEP); // update ti
      if (stop) return txt_size;
#endif
    }
  } // end while

  return txt_size;
};

#endif
