// soskm.h -- The -*- C++ -*- Shift Or skm interface.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _SOSKM_H_
#define _SOSKM_H_

#include "skm.h"
#include <vector> // skip table

// ===================================
// SO SKM (Shift Or) : Class Interface
// ===================================

class so_skm : public skm_algo {

protected:

  // internal variables
  vector<unsigned int> mask;       // mask vector
  unsigned int         state;      // state of the search
  unsigned int         max_states; // max number of states
  unsigned int         last_state; // last possible state

  skm_patt::iterator   pmpi;       // Partial match pattern iterator
  astring::iterator    pmti;       // Partial match text iterator

public:

  // constructor / destructor
  so_skm() : skm_algo() {};
  so_skm(const alpha &a) : skm_algo(a) {};

  ~so_skm() {};

  // long algorithm name
  virtual 
  const char * name() const { return "Shift Or"; };

  // short algorithm name
  virtual
  const char * id() const { return "so"; };

  // print preprocesed data 
  virtual 
  ostream & print_pdata(ostream &o) const;

protected:

  virtual 
  void preprocess ();                // preprocess the pattern

  virtual 
  void run_preprocessor();           // run the preprocessor

  virtual 
  astring::size_type match();       // match the pattern

  virtual
  astring::size_type run_matcher(); // run the matcher

  virtual
  void algorithm_step(breakpoint WHERE, int count);
  // algorithm step function

};

// STATIC OBJECTS
extern so_skm soskm;

#endif
