// sqsskm.cpp -- The -*- C++ -*- Sunday Quick Search skm prep & matcher. 
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifdef SQSSKM_CC

// Sunday Quick Search preprocessor
void sqs_skm::PREPROCESSOR () {

  // internal variables
  skm_patt::size_type i;

  // INIT SKIP TABLE

  // clear skip vector
  skip.erase(skip.begin(), skip.end());            // erase previous values
  skip.reserve(alpha_size);                        // inform of space needed

  // allocate vector with assignment (skip[i] = pat_size + 1)
  skip.insert(skip.begin(), alpha_size, pat_size + 1); 

  // compute skip & pat_last
  pat_last = pat_beg;

#ifdef SQSSKM_RUN
  report_step(INIT_PREPROCESSOR); // Step 0
  if (stop) return;

  report_step(PREPROCESSOR_STEP, alpha_size);
  // Inicialization of skip -> no psteps, but they are there
  if (stop) return;
#endif

  for (i= pat_size; i > 0; i--) {
    skip [ aindex(*pat_last) ] = i;
#ifdef SQSSKM_RUN
    report_step(PREPROCESSOR_STEP); // skip updates
    if (stop) return;
#endif
    pat_last++;
  }

  pat_last--;

  /*
     Now we have:

      skip[pat[i-1]] == pat_size + 1 - i   
      pat_last       == pat_end - 1
  */
};

//  Sunday Quick Search matcher
astring::size_type sqs_skm::MATCHER (){

  // LOCAL OPTIMIZATIONS


  // USE p0 INSTEAD OF *pat_beg 
  register skm_patt::value_type p0 = *pat_beg;  

  // USE ti_last INSTEAD OF txt_end - ti ON while LOOP
  register astring::iterator   ti_last = txt_end - pat_size;

#ifndef SQSSKM_RUN
  // Redefine the text iterator declaring it in register
  register astring::iterator   ti = this->ti;
#endif

#ifdef SQSSKM_RUN
  report_step(INIT_MATCHER); // Step 0, get first text positions
  if (stop) return txt_size;
#endif

  while (ti < ti_last) { 

    // Do partial match if current text symbol == last pattern symbol
    if (*ti==p0) {

      pmti = ti;
      pmpi = pat_beg;

#ifdef SQSSKM_RUN
      report_step(PARTIAL_MATCH); // current text symbol = current patt symbol
      if (stop) return txt_size;
#endif

      // Use do-while instead of while to allow patterns of size one.
      do {

	pmpi++; pmti++;

	// If pattern matched apply match function
	if (pmpi == pat_end) {

	  // Internal variables
	  astring::size_type midx; // match index 

	  midx = ti - txt_beg;

#ifdef SQSSKM_RUN
	  report_step(FULL_MATCH);
	  if (stop) return txt_size;
#endif

    	  if (report_match(midx)) {
	    return (midx);                // ... return the match
	  } else {
#ifdef SQSSKM_RUN
	    report_step(MATCH_RECOVERY);
	    if (stop) return txt_size;
#endif
	    break;                        // ... and continue searching
	  } // end if

	} // end if

#ifdef SQSSKM_RUN
	report_step(PARTIAL_MATCH_STEP); // advance pattern & text
	if (stop) return txt_size;
#endif

      } while (*pmpi==*pmti);

    } // end if

    ti += skip[ aindex(*(ti + pat_size)) ];

#ifdef SQSSKM_RUN
    report_step(MATCHER_STEP); // advance text 
    if (stop) return txt_size;
#endif

  } // end while

  return (txt_size);
};

#endif
