// sqsskm.h -- The -*- C++ -*- Sunday Quick Search skm interface.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _SQSSKM_H_
#define _SQSSKM_H_

#include "skm.h"
#include <vector> // skip table

// ===============================================
// SQS SKM (Sunday Quick Search) : Class Interface
// ===============================================

class sqs_skm : public skm_algo {

protected:

  // internal variables
  vector<astring::size_type> skip;     // Skip Table

  skm_patt::iterator         pat_last; // Last pattern symbol iterator

  skm_patt::iterator         pmpi;     // Partial match pattern iterator
  astring::iterator          pmti;     // Partial match text iterator

public:

  // constructor / destructor
  sqs_skm() : skm_algo() {};
  sqs_skm(const alpha &a) : skm_algo(a) {};

  ~sqs_skm() {};

  // long algorithm name
  virtual 
  const char * name() const { return "Sunday Quick Search"; };

  // short algorithm name
  virtual 
  const char * id() const { return "sqs"; };

  // print preprocesed data 
  virtual 
  ostream & print_pdata(ostream &o) const;

protected:

  virtual 
  void preprocess ();                // preprocess the pattern

  virtual 
  void run_preprocessor();           // run the preprocessor

  virtual 
  astring::size_type match();       // match the pattern

  virtual
  astring::size_type run_matcher(); // run the matcher

  virtual
  void algorithm_step(breakpoint WHERE, int count);
  // algorithm step function
};

// STATIC OBJECTS
extern sqs_skm sqsskm;

#endif
