// stepacct.h -- The -*- C++ -*- step accounting class interface.
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _STEPACCT_H_
#define _STEPACCT_H_

/**

  DESCRIPTION
  
  This is an auxiliary class to do step accounting (kind of breakpoints)
  
**/

#include <iostream.h>

// ===========================
// STEP ACCT : Class Interface
// ===========================

class stepacct {

protected:

  const char **names; // Step names
  int        *steps;  // Number of steps by breakpoint
  const int  size;    // Total number of steps

public:

  stepacct(const char *sn[], int ns) : names(sn), size(ns) {
    steps = new int[ns];
    reset();
  };

  stepacct(const stepacct &sa) : names(sa.names), size(sa.size) {
    steps = new int[size];

    for(int i=0; i < size; i++)
      steps[i]= sa.steps[i];    
  };

  ~stepacct() {
    delete []steps;
  };

  // Set the value of the counter to count
  inline
  void set(int WHERE, int count = 0);

  // Update step count
  inline
  void update(int WHERE, int count = 1);

  // Reset step accounting
  inline
  void reset();

  // Return the number of steps through the breakpoint
  inline 
  int nsteps(int WHERE) const;

  // Return the breakpoint name
  inline
  const char *bpname(int WHERE) const;

  // Return the number of breakpoints
  inline
  int nbps() const;

  // Output
  friend 
  ostream & operator<< (ostream &o, const stepacct &sa);
};

// ==========================
// STEP ACCT : Inline Methods
// ==========================

// Set the value of the counter to count
inline
void stepacct::set (int WHERE, int count) {
  steps[WHERE] = count;
};

// Update step count
inline
void stepacct::update (int WHERE, int count) {
  steps[WHERE] += count;
};

// Reset step accounting
inline
void stepacct::reset(){
  for(int i=0; i < size; i++)
    steps[i]= 0;
};

// Return the number of steps through the breakpoint
inline 
int stepacct::nsteps(int WHERE) const {
  return steps[WHERE];  
}

// Return the breakpoint name
inline
const char *stepacct::bpname(int WHERE) const {
  return names[WHERE];
}

// Return the number of breakpoints
inline
int stepacct::nbps() const {
  return size;
};

#endif
