// stopw.h -- The -*- C++ -*- Stopwatch class interface
// Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>
// This file is part of the C++ Alphabetic Data Management Library.

#ifndef _STOPW_H_
#define _STOPW_H_

/** 

  DESCRIPTION

  Stopwatch to measure laptimes in a program

  This StopWatch has special versions defined for the MacOS and the
  DJGPP DOS compiler, a Windows version was also defined, but I've
  removed it because I haven't tested it.

  A tms UNIX version can be defined, but in GNU systems it reports the
  same value than clock().

**/

#include "system.h"

#if defined(__macos__)
# include <Timer.h>
# include <string.h> // memset
#else
# include <time.h>
#endif

// ===========================
// StopWatch : Class Interface
// ===========================

class stopwatch {

public:

  inline
  stopwatch(); 
  
  inline
  ~stopwatch();

  inline
  void   start();  

  inline
  double stop();

  inline
  double resolution ();

private:

#if defined(__macos__)
  TMTask        myTMT;  
  unsigned long delay;
  bool          running;
#elif defined(__djgpp__)
  uclock_t      start_time;
#else
  clock_t       start_time;
#endif
};

extern stopwatch timer;

// ==========================
// StopWatch : Inline Methods
// ==========================

#if defined(__macos__) // MacOS stopwatch

inline
stopwatch::stopwatch() 
  : delay(2100*1000*1000), running(false) {};
    // 35 minutes of delay

inline
stopwatch::~stopwatch(){
  if (running) stop();
};

inline
void stopwatch::start(){
  if (running)
    stop();

  memset (&myTMT, 0, sizeof(myTMT));    // clear structure
  InsTime((QElemPtr)&myTMT);            // Insert null TMTask
  PrimeTime((QElemPtr)&myTMT, -delay);  // Schedule

  running = true;
}

inline
double stopwatch::stop() {
  running = false;
  RmvTime((QElemPtr)&myTMT);
  return double(delay + myTMT.tmCount);
}

inline
double stopwatch::resolution () {
  return 1.0e6; // microseconds
};

#elif defined(__djgpp__) // DOS stopwatch

inline
stopwatch::stopwatch() {};

inline
stopwatch::~stopwatch(){};

inline  
void stopwatch::start(){ 
  start_time = uclock();
};

inline 
double stopwatch::stop(){
  return double(uclock() - start_time);
};

inline
double stopwatch::resolution () {
  return double(UCLOCKS_PER_SEC);
};

#else // ANSI stopwatch

inline
stopwatch::stopwatch() {};

inline
stopwatch::~stopwatch(){};

inline  
void stopwatch::start(){ 
  start_time = clock();
};

inline 
double stopwatch::stop(){
  return double(clock() - start_time);
};

inline
double stopwatch::resolution () {
  return double(CLOCKS_PER_SEC);
};

#endif

#endif


