/*  input.cc -- Main test program for the -*- C++ -*- ADML
    Copyright (C) 1997 Sergio Talens-Oliag <sto@uv.es>

    This file is part of the C++ Alphabetic Data Management Library. */

#include "skmtest.h"

#include <stdio.h>  // file operations

// We use buffers and the C functions for reading files because of the
// inefficient Metrowerks MSL C++ 1.0 'getline' implementation.  It reads
// characters one by one !!!

// ===============
// Input Functions
// ===============

char parse_char (const char *cb, char **ce) {

  char          cr;
  unsigned char ch;
  bool          hex;

  *ce= (char *) cb; 
  cr = **ce;

  if (cr != '\\')
    return *cb;

  hex = false;
  (*ce)++; cr = **ce;

  switch (cr) {
    case 'n': return '\n';
    case 't': return '\t';
    case 'v': return '\v';
    case 'b': return '\b';
    case 'r': return '\r';
    case 'f': return '\f';
    case 'a': return '\a';
    case '\\': return '\\';
    case '\?': return '\?';
    case '\'': return '\'';
    case '\"': return '\"';
    case 'x' : ch = 0; hex = true; break;
    case '0' : ch = 0; break;
    case '1' : ch = 1; break;
    case '2' : ch = 2; break;
    case '3' : ch = 3; break;
    case '4' : ch = 4; break;
    case '5' : ch = 5; break;
    case '6' : ch = 6; break;
    case '7' : ch = 7; break;
    default  : return cr;
  }

  for (int i=0; i<2; i++) {
    (*ce)++; cr = **ce;
    if (!hex) {
      switch (cr) {
        case '0' : ch = ch * 8 + 0; break;
        case '1' : ch = ch * 8 + 1; break;
        case '2' : ch = ch * 8 + 2; break;
        case '3' : ch = ch * 8 + 3; break;
        case '4' : ch = ch * 8 + 4; break;
        case '5' : ch = ch * 8 + 5; break;
        case '6' : ch = ch * 8 + 6; break;
        case '7' : ch = ch * 8 + 7; break;
        default: break;
      }
    } else {
      switch (cr) {
        case '0' : ch = ch * 16 + 0; break;
        case '1' : ch = ch * 16 + 1; break;
        case '2' : ch = ch * 16 + 2; break;
        case '3' : ch = ch * 16 + 3; break;
        case '4' : ch = ch * 16 + 4; break;
        case '5' : ch = ch * 16 + 5; break;
        case '6' : ch = ch * 16 + 6; break;
        case '7' : ch = ch * 16 + 7; break;
        case '8' : ch = ch * 16 + 8; break;
        case '9' : ch = ch * 16 + 9; break;
	case 'A' : case 'a' : ch = ch * 16 + 10; break;
	case 'B' : case 'b' : ch = ch * 16 + 11; break;
	case 'C' : case 'c' : ch = ch * 16 + 12; break;
	case 'D' : case 'd' : ch = ch * 16 + 13; break;
	case 'E' : case 'e' : ch = ch * 16 + 14; break;
	case 'F' : case 'f' : ch = ch * 16 + 15; break;
        default: break;
      }
    }
  }
 
  return ch;
};

// Read PATTERN and verify it's alphabetic
bool read_patt (const char *s) {
 char   ch;
 char   * sp = (char *) s;
 
 pat = "";
 while (*sp) {
   ch = parse_char(sp, &sp);
   if (al.belongs(ch))
     pat += ch;
   else {
     return false; // pattern is not alphabetic
   }
   sp++;
 }
 return true;
}

// Read PATTERNS from file
bool read_pattv (const char *fn) {

  const size_t       BUFF_SIZE= 1024; // 1K buffer

  FILE               *ipf;
  int                ich;
  unsigned char      ch;
  skm_patt           ps;
  bool               reading;

  // Try to open PATTERNS file
  ipf = fopen(fn, "r");

  if (!ipf) 
    return false;

  reading = true;

  while (reading) {

    ich = fgetc(ipf);

    if (ich == EOF)
      break;

    ch = ich;

    if (ch == '\n') {
      if (ps.size() > 0) {
	pv.push_back(ps);
        ps = "";
      }
    } else if (al.belongs(ch)) {
      ps += ch;
    } else {
      ps = "";
      while (true) {
        ich = fgetc(ipf);
        if (ich == EOF) {
	  reading = false;
          break;
        }
        if (ich == '\n')
          break;
      }
    }
  }

  fclose(ipf);
  
  if (pv.size() != 0)
    return true;
  else
    return false;
}

bool read_text () {

  FILE  *itf;
  const size_t   BUFF_SIZE= 16 * 1024; // 16K buffer
  char  txt_buf [BUFF_SIZE];
  int   bytes_read;
  bool  reading;
  
  // Try to open TEXT file
  itf = fopen(txt_filename, "r");

  if (!itf) return false;

  // Read all TEXT as a single string -- should do some error checking
  reading = true;
  while (reading){
    bytes_read = fread ((void *)txt_buf, sizeof(char), BUFF_SIZE, itf);
    if (bytes_read > 0)
      txt.append((char *)txt_buf, bytes_read);
    else 
      reading=false;
  };

  // Close the TEXT file
  fclose(itf);

  return true;
}
